@@ -46,8 +46,8 @@
 use std::collections::HashMap;
 use std::io;
 
-use super::ops_hnsw::{Hnsw, Layer};
 use super::ram_hnsw::{Edge, EntryPoint, RAMHnsw, RAMLayer};
+use super::search::{SearchableHnsw, SearchableLayer};
 use crate::VectorAddr;
 use crate::data_types::usize_utils::*;
 
@@ -66,7 +66,7 @@ pub struct DiskLayer<'a> {
     layer: usize,
 }
 
-impl<'a> Layer for &'a DiskLayer<'a> {
+impl<'a> SearchableLayer for &'a DiskLayer<'a> {
     type EdgeIt = EdgeIter<'a>;
     fn get_out_edges(&self, address: VectorAddr) -> Self::EdgeIt {
         let node = DiskHnsw::get_node(self.hnsw, address);
@@ -74,7 +74,7 @@ impl<'a> Layer for &'a DiskLayer<'a> {
     }
 }
 
-impl<'a> Layer for DiskLayer<'a> {
+impl<'a> SearchableLayer for DiskLayer<'a> {
     type EdgeIt = EdgeIter<'a>;
     fn get_out_edges(&self, address: VectorAddr) -> Self::EdgeIt {
         let node = DiskHnsw::get_node(self.hnsw, address);
@@ -82,9 +82,9 @@ impl<'a> Layer for DiskLayer<'a> {
     }
 }
 
-impl<'a> Hnsw for &'a [u8] {
+impl<'a> SearchableHnsw for &'a [u8] {
     type L = DiskLayer<'a>;
-    fn get_entry_point(&self) -> Option<EntryPoint> {
+    fn get_entry_point(&self) -> EntryPoint {
         DiskHnsw::get_entry_point(self)
     }
     fn get_layer(&self, i: usize) -> Self::L {
@@ -124,14 +124,16 @@ impl DiskHnsw {
         let mut length = offset;
         let mut indexing = HashMap::new();
         for layer in 0..hnsw.no_layers() {
-            let no_edges = hnsw.get_layer(layer).no_out_edges(node);
+            let num_edges = hnsw.get_layer(layer).num_out_edges(&node);
             indexing.insert(layer, length);
-            buf.write_all(&no_edges.to_le_bytes())?;
+            buf.write_all(&num_edges.to_le_bytes())?;
             length += USIZE_LEN;
-            for (cnx, edge) in hnsw.get_layer(layer).get_out_edges(node) {
-                buf.write_all(&(cnx.0 as usize).to_le_bytes())?;
-                buf.write_all(&edge.to_le_bytes())?;
-                length += CNX_LEN;
+            if num_edges > 0 {
+                for (cnx, edge) in hnsw.get_layer(layer).get_out_edges(node) {
+                    buf.write_all(&(cnx.0 as usize).to_le_bytes())?;
+                    buf.write_all(&edge.to_le_bytes())?;
+                    length += CNX_LEN;
+                }
             }
         }
         for layer in (0..hnsw.no_layers()).rev() {
@@ -158,38 +160,40 @@ impl DiskHnsw {
         }
     }
     pub fn serialize_into<W: io::Write>(mut buf: W, no_nodes: u32, hnsw: RAMHnsw) -> io::Result<()> {
-        if let Some(entry_point) = hnsw.entry_point {
-            let mut length = 0;
-            let mut nodes_end = vec![];
-            for node in 0..no_nodes {
-                length = DiskHnsw::serialize_node(&mut buf, length, node, &hnsw)?;
-                nodes_end.push(length)
-            }
-            for ends_at in nodes_end.into_iter().rev() {
-                buf.write_all(&ends_at.to_le_bytes())?;
-                length += USIZE_LEN;
-            }
-            let EntryPoint { node, layer } = entry_point;
-            buf.write_all(&layer.to_le_bytes())?;
-            buf.write_all(&(node.0 as usize).to_le_bytes())?;
-            let _length = length + 2 * USIZE_LEN;
-            buf.flush()?;
+        if no_nodes == 0 {
+            // Empty graph, nothing to serialize
+            return Ok(());
         }
+
+        let mut length = 0;
+        let mut nodes_end = vec![];
+        for node in 0..no_nodes {
+            length = DiskHnsw::serialize_node(&mut buf, length, node, &hnsw)?;
+            nodes_end.push(length)
+        }
+        for ends_at in nodes_end.into_iter().rev() {
+            buf.write_all(&ends_at.to_le_bytes())?;
+            length += USIZE_LEN;
+        }
+        let EntryPoint { node, layer } = hnsw.entry_point;
+        buf.write_all(&layer.to_le_bytes())?;
+        buf.write_all(&(node.0 as usize).to_le_bytes())?;
+        let _length = length + 2 * USIZE_LEN;
+        buf.flush()?;
+
         Ok(())
     }
     // hnsw must be serialized using DiskHnsw, may have trailing bytes at the start.
-    pub fn get_entry_point(hnsw: &[u8]) -> Option<EntryPoint> {
-        if !hnsw.is_empty() {
-            let node_start = hnsw.len() - USIZE_LEN;
-            let layer_start = node_start - USIZE_LEN;
-            let node_addr = usize_from_slice_le(&hnsw[node_start..(node_start + NODE_LEN)]);
-            let layer = usize_from_slice_le(&hnsw[layer_start..(layer_start + USIZE_LEN)]);
-            Some(EntryPoint {
-                node: VectorAddr(node_addr as u32),
-                layer,
-            })
-        } else {
-            None
+    pub fn get_entry_point(hnsw: &[u8]) -> EntryPoint {
+        assert!(!hnsw.is_empty());
+
+        let node_start = hnsw.len() - USIZE_LEN;
+        let layer_start = node_start - USIZE_LEN;
+        let node_addr = usize_from_slice_le(&hnsw[node_start..(node_start + NODE_LEN)]);
+        let layer = usize_from_slice_le(&hnsw[layer_start..(layer_start + USIZE_LEN)]);
+        EntryPoint {
+            node: VectorAddr(node_addr as u32),
+            layer,
         }
     }
     // hnsw must be serialized using MHnsw, may have trailing bytes at the start.
@@ -231,7 +235,12 @@ impl DiskHnsw {
                 let cnx_end = cnx_start + number_edges * CNX_LEN;
 
                 if number_edges > 0 {
-                    let ram_edges = ram.layers[layer_index].out.entry(VectorAddr(node_index)).or_default();
+                    let mut ram_edges = ram.layers[layer_index]
+                        .out
+                        .entry(VectorAddr(node_index))
+                        .or_default()
+                        .write()
+                        .unwrap();
                     let edges = EdgeIter {
                         crnt: 0,
                         buf: &hnsw[cnx_start..cnx_end],
@@ -259,9 +268,11 @@ impl DiskHnsw {
 
 #[cfg(test)]
 mod tests {
+    use std::sync::RwLock;
+
     use super::*;
     use crate::hnsw::ram_hnsw::RAMLayer;
-    fn layer_check<L: Layer>(buf: L, no_nodes: u32, cnx: &[Vec<(VectorAddr, Edge)>]) {
+    fn layer_check<L: SearchableLayer>(buf: L, no_nodes: u32, cnx: &[Vec<(VectorAddr, Edge)>]) {
         let no_cnx = vec![];
         for i in 0..no_nodes {
             let expected = cnx.get(i as usize).unwrap_or(&no_cnx);
@@ -274,8 +285,7 @@ mod tests {
         let hnsw = RAMHnsw::new();
         let mut buf = vec![];
         DiskHnsw::serialize_into(&mut buf, 0, hnsw).unwrap();
-        let ep = DiskHnsw::get_entry_point(&buf);
-        assert_eq!(ep, None);
+        assert!(buf.is_empty());
     }
 
     #[test]
@@ -290,7 +300,7 @@ mod tests {
             out: cnx0
                 .iter()
                 .enumerate()
-                .map(|(i, c)| (VectorAddr(i as u32), c.clone()))
+                .map(|(i, c)| (VectorAddr(i as u32), RwLock::new(c.clone())))
                 .collect(),
         };
         let cnx1 = vec![vec![(VectorAddr(1), 4.0)], vec![(VectorAddr(2), 5.0)]];
@@ -298,7 +308,7 @@ mod tests {
             out: cnx1
                 .iter()
                 .enumerate()
-                .map(|(i, c)| (VectorAddr(i as u32), c.clone()))
+                .map(|(i, c)| (VectorAddr(i as u32), RwLock::new(c.clone())))
                 .collect(),
         };
         let cnx2 = vec![vec![(VectorAddr(1), 6.0)]];
@@ -306,7 +316,7 @@ mod tests {
             out: cnx2
                 .iter()
                 .enumerate()
-                .map(|(i, c)| (VectorAddr(i as u32), c.clone()))
+                .map(|(i, c)| (VectorAddr(i as u32), RwLock::new(c.clone())))
                 .collect(),
         };
         let entry_point = EntryPoint {
@@ -314,11 +324,11 @@ mod tests {
             layer: 2,
         };
         let mut hnsw = RAMHnsw::new();
-        hnsw.entry_point = Some(entry_point);
+        hnsw.entry_point = entry_point;
         hnsw.layers = vec![layer0, layer1, layer2];
         let mut buf = vec![];
         DiskHnsw::serialize_into(&mut buf, no_nodes, hnsw).unwrap();
-        let ep = DiskHnsw::get_entry_point(&buf).unwrap();
+        let ep = DiskHnsw::get_entry_point(&buf);
         assert_eq!(ep, entry_point);
         let layer0 = buf.as_slice().get_layer(0);
         layer_check(layer0, no_nodes, &cnx0);
@@ -340,7 +350,7 @@ mod tests {
             out: cnx0
                 .iter()
                 .enumerate()
-                .map(|(i, c)| (VectorAddr(i as u32), c.clone()))
+                .map(|(i, c)| (VectorAddr(i as u32), RwLock::new(c.clone())))
                 .collect(),
         };
         let cnx1 = [vec![(VectorAddr(1), 4.0)], vec![(VectorAddr(2), 5.0)]];
@@ -348,7 +358,7 @@ mod tests {
             out: cnx1
                 .iter()
                 .enumerate()
-                .map(|(i, c)| (VectorAddr(i as u32), c.clone()))
+                .map(|(i, c)| (VectorAddr(i as u32), RwLock::new(c.clone())))
                 .collect(),
         };
         let cnx2 = [vec![(VectorAddr(1), 6.0)]];
@@ -356,7 +366,7 @@ mod tests {
             out: cnx2
                 .iter()
                 .enumerate()
-                .map(|(i, c)| (VectorAddr(i as u32), c.clone()))
+                .map(|(i, c)| (VectorAddr(i as u32), RwLock::new(c.clone())))
                 .collect(),
         };
         let entry_point = EntryPoint {
@@ -364,7 +374,7 @@ mod tests {
             layer: 2,
         };
         let mut hnsw = RAMHnsw::new();
-        hnsw.entry_point = Some(entry_point);
+        hnsw.entry_point = entry_point;
         hnsw.layers = vec![layer0, layer1, layer2];
         let mut buf = vec![];
         DiskHnsw::serialize_into(&mut buf, no_nodes, hnsw).unwrap();
