@@ -24,13 +24,15 @@ mod tests;
 use crate::config::{VectorConfig, flags};
 use crate::data_store::{DataStore, DataStoreV1, DataStoreV2, OpenReason, ParagraphRef, VectorRef};
 use crate::formula::Formula;
-use crate::hnsw::*;
 use crate::inverted_index::{FilterBitSet, InvertedIndexes, build_indexes};
 use crate::vector_types::rabitq;
-use crate::{ParagraphAddr, VectorAddr, VectorErr, VectorR, VectorSegmentMeta, VectorSegmentMetadata};
+use crate::{ParagraphAddr, VectorAddr, VectorErr, VectorR, VectorSegmentMeta, VectorSegmentMetadata, hnsw};
+use crate::{hnsw::*, inverted_index};
 use core::f32;
 use io::{BufWriter, Write};
-use memmap2::Mmap;
+use memmap2::{Mmap, MmapOptions};
+use rayon::prelude::*;
+
 use std::cmp::Reverse;
 use std::collections::HashSet;
 use std::fs::File;
@@ -40,17 +42,17 @@ use std::path::Path;
 use std::time::Instant;
 use tracing::{debug, trace};
 
-/// How much expensive is to find a node via HNSW compared to a simple brute force scan
-const HNSW_COST_FACTOR: usize = 100;
-
 mod file_names {
     pub const HNSW: &str = "index.hnsw";
 }
 
 pub fn open(metadata: VectorSegmentMetadata, config: &VectorConfig) -> VectorR<OpenSegment> {
+    // TODO: we should get this flag from the VectorConfig or some other place
+    let prewarm = false;
+
     let path = &metadata.path;
     let data_store: Box<dyn DataStore> = if DataStoreV1::exists(path)? {
-        let data_store = DataStoreV1::open(path, &config.vector_type, OpenReason::Search)?;
+        let data_store = DataStoreV1::open(path, &config.vector_type, OpenReason::Search { prewarm })?;
         // Build the index at runtime if they do not exist. This can
         // be removed once we have migrated all existing indexes
         if !InvertedIndexes::exists(path) {
@@ -58,7 +60,7 @@ pub fn open(metadata: VectorSegmentMetadata, config: &VectorConfig) -> VectorR<O
         }
         Box::new(data_store)
     } else {
-        let data_store = DataStoreV2::open(path, &config.vector_type, OpenReason::Search)?;
+        let data_store = DataStoreV2::open(path, &config.vector_type, OpenReason::Search { prewarm })?;
         // Build the index at runtime if they do not exist. This can
         // be removed once we have migrated all existing indexes
         if !InvertedIndexes::exists(path) {
@@ -66,9 +68,13 @@ pub fn open(metadata: VectorSegmentMetadata, config: &VectorConfig) -> VectorR<O
         }
         Box::new(data_store)
     };
-    let hnsw_file = File::open(path.join(file_names::HNSW))?;
 
-    let index = unsafe { Mmap::map(&hnsw_file)? };
+    let hnsw_file = File::open(path.join(file_names::HNSW))?;
+    let mut index_options = MmapOptions::new();
+    if prewarm {
+        index_options.populate();
+    }
+    let index = unsafe { index_options.map(&hnsw_file)? };
 
     // Telling the OS our expected access pattern
     #[cfg(not(target_os = "windows"))]
@@ -77,7 +83,7 @@ pub fn open(metadata: VectorSegmentMetadata, config: &VectorConfig) -> VectorR<O
         index.advise(memmap2::Advice::WillNeed)?;
     }
 
-    let inverted_indexes = InvertedIndexes::open(path, metadata.records)?;
+    let inverted_indexes = InvertedIndexes::open(path, metadata.records, inverted_index::OpenOptions { prewarm })?;
     let alive_bitset = FilterBitSet::new(metadata.records, true);
 
     Ok(OpenSegment {
@@ -149,16 +155,18 @@ fn merge_indexes<DS: DataStore + 'static>(
         // If there are no deletions, we can reuse the first segment
         // HNSW since its indexes will match the the ones in data_store
         index = DiskHnsw::deserialize(&operants[0].index);
+        index.fix_broken_graph();
         start_vector_index = operants[0].data_store.stored_vector_count();
     }
     let merged_vectors_count = data_store.stored_vector_count();
 
     // Creating the hnsw for the new node store.
     let retriever = Retriever::new(&data_store, config, -1.0);
-    let mut ops = HnswOps::new(&retriever, false);
-    for id in start_vector_index..merged_vectors_count {
-        ops.insert(VectorAddr(id), &mut index);
-    }
+    let mut builder = HnswBuilder::new(&retriever);
+    builder.initialize_graph(&mut index, start_vector_index, merged_vectors_count);
+    (start_vector_index..merged_vectors_count)
+        .into_par_iter()
+        .for_each(|id| builder.insert(VectorAddr(id), &index));
 
     let hnsw_path = segment_path.join(file_names::HNSW);
     let mut hnsw_file = File::options()
@@ -180,8 +188,6 @@ fn merge_indexes<DS: DataStore + 'static>(
         index.advise(memmap2::Advice::Random)?;
     }
 
-    build_indexes(segment_path, &data_store)?;
-
     let metadata = VectorSegmentMetadata {
         path: segment_path.to_path_buf(),
         records: data_store.stored_paragraph_count() as usize,
@@ -191,7 +197,11 @@ fn merge_indexes<DS: DataStore + 'static>(
     };
 
     build_indexes(segment_path, &data_store)?;
-    let inverted_indexes = InvertedIndexes::open(segment_path, merged_vectors_count as usize)?;
+    let inverted_indexes = InvertedIndexes::open(
+        segment_path,
+        merged_vectors_count as usize,
+        inverted_index::OpenOptions { prewarm: false },
+    )?;
     let alive_bitset = FilterBitSet::new(metadata.records, true);
 
     Ok(OpenSegment {
@@ -262,10 +272,11 @@ fn create_indexes<DS: DataStore + 'static>(
     // Creating the HNSW using the mmaped nodes
     let mut index = RAMHnsw::new();
     let retriever = Retriever::new(&data_store, config, -1.0);
-    let mut ops = HnswOps::new(&retriever, false);
-    for id in 0..vector_count {
-        ops.insert(VectorAddr(id), &mut index)
-    }
+    let mut builder = HnswBuilder::new(&retriever);
+    builder.initialize_graph(&mut index, 0, vector_count);
+    (0..vector_count)
+        .into_par_iter()
+        .for_each(|id| builder.insert(VectorAddr(id), &index));
 
     {
         // The HNSW is on RAM
@@ -291,7 +302,11 @@ fn create_indexes<DS: DataStore + 'static>(
     };
 
     build_indexes(path, &data_store)?;
-    let inverted_indexes = InvertedIndexes::open(path, vector_count as usize)?;
+    let inverted_indexes = InvertedIndexes::open(
+        path,
+        vector_count as usize,
+        inverted_index::OpenOptions { prewarm: false },
+    )?;
     let alive_bitset = FilterBitSet::new(metadata.records, true);
 
     Ok(OpenSegment {
@@ -525,23 +540,23 @@ impl OpenSegment {
         if matching == 0 {
             return Box::new(empty());
         }
-        let expected_traversal_scan = top_k * self.metadata.records / matching;
 
         let t = Instant::now();
         let method;
-        let results = if matching < expected_traversal_scan * HNSW_COST_FACTOR {
-            method = "brute force";
-            self.brute_force_search(bitset, top_k, retriever, encoded_query, &raw_query)
-        } else {
+        let results = if use_hnsw(self.metadata.records, matching, top_k, rabitq) {
             method = "hnsw";
-            let ops = HnswOps::new(&retriever, true);
-            let neighbours = ops.search(encoded_query, self.index.as_ref(), top_k, bitset, with_duplicates);
+            let ops = HnswSearcher::new(&retriever, true);
+            let filter = NodeFilter::new(bitset, with_duplicates, config);
+            let neighbours = ops.search(encoded_query, self.index.as_ref(), top_k, filter);
             Box::new(
                 neighbours
                     .into_iter()
                     .map(|(address, dist)| ScoredVector::new(address, self.data_store.as_ref(), dist))
                     .take(top_k),
             )
+        } else {
+            method = "brute force";
+            self.brute_force_search(bitset, top_k, retriever, encoded_query, &raw_query)
         };
 
         let time = t.elapsed();
@@ -608,6 +623,42 @@ impl OpenSegment {
     }
 }
 
+fn use_hnsw(total_nodes: usize, matching_nodes: usize, top_k: usize, has_rabitq: bool) -> bool {
+    // Cost of a full vector comparison compared to a quantized vector comparison
+    let full_cost: usize;
+    // How many more vectors are evaluated in layer_search for quantized vectors
+    let search_mult: usize;
+    // How many vectors are reranked
+    let rerank_mult: usize;
+    if has_rabitq {
+        full_cost = 16;
+        search_mult = rabitq::RERANKING_FACTOR * 3 / 4;
+        rerank_mult = rabitq::RERANKING_FACTOR / 2;
+    } else {
+        full_cost = 1;
+        search_mult = 1;
+        rerank_mult = 0;
+    }
+
+    // Estimated vectors visited during hnsw search (quantized vectors)
+    let hnsw_rq = ((total_nodes as f32).ln() - 2.0).powi(2) * (top_k as f32).ln() * search_mult as f32;
+    // Estimated vectors visited in closest_up_nodes (full vectors)
+    let hnsw_full = (top_k * rerank_mult) + (top_k * hnsw::M * total_nodes / matching_nodes);
+
+    // Quantized vectors evaluated with brute-force
+    let bf_rq = matching_nodes;
+    // Estimated full vectors to be evaluated during rerank in brute-force
+    let bf_full = top_k * rerank_mult;
+
+    let hnsw_cost = hnsw_rq as usize + hnsw_full * full_cost;
+    let bf_cost = bf_rq + bf_full * full_cost;
+
+    let use_hnsw = hnsw_cost < bf_cost;
+    debug!(hnsw_cost, bf_cost, use_hnsw, "Estimated search costs");
+
+    use_hnsw
+}
+
 #[cfg(test)]
 mod test {
     use std::collections::{BTreeMap, HashSet};
