@@ -279,7 +279,7 @@ pub async fn delete_resource(meta: &NidxMetadata, shard_id: &str, resource: Stri
 
     let mut tx = meta.transaction().await?;
     for index in indexes {
-        Deletion::create(&mut *tx, index.id, seq, &[resource.clone()]).await?;
+        Deletion::create(&mut *tx, index.id, seq, std::slice::from_ref(&resource)).await?;
         Index::updated(&mut *tx, &index.id).await?;
     }
     tx.commit().await?;
@@ -421,7 +421,7 @@ mod tests {
     use std::io::{Seek, Write};
 
     use nidx_protos::StringList;
-    use nidx_vector::config::{Similarity, VectorConfig, VectorType};
+    use nidx_vector::config::{Similarity, VectorCardinality, VectorConfig, VectorType};
     use tempfile::tempfile;
     use uuid::Uuid;
 
@@ -434,6 +434,7 @@ mod tests {
         normalize_vectors: false,
         vector_type: VectorType::DenseF32 { dimension: 3 },
         flags: vec![],
+        vector_cardinality: VectorCardinality::Single,
     };
 
     #[sqlx::test]
